/*
Copyright (C) 2017 Hayakuchi

This file is part of HinesmPaint.

HinesmPaint is free software.
You can redistribute HinesmPaint and/or modify it under the GPL Licence version3 and LGPL Licence version 3.
(GPL text and LGPL text in http://www.gnu.org/licenses/)

このプログラムはフリーソフトです。
あなたはGPLライセンスversion3及びLGPLライセンスversion3の元でこのプログラムを改変、再配布することが可能です。
(GPLとLGPLの内容は http://www.gnu.org/licenses/)にあります)
*/
package hinesm.gazoushori.special;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import jpen.event.PenListener;
import jpen.owner.multiAwt.AwtPenToolkit;
import jpen.PKindEvent;
import jpen.PLevelEvent;
import jpen.PButtonEvent;
import jpen.PScrollEvent;
import jpen.PLevel;

import hinesm.gazoushori.gazoushori;
import hinesm.gazoushori.ConfigFileRW;
import hinesm.imagechenge.ImageChenge;
import hinesm.imagechenge.DrawLineInfo;
import hinesm.util.gazobutton.GazoButton;
import hinesm.util.gazobutton.ScrollBar;

public class CanvasPanel extends GazoButton implements PenListener
{
public enum FILTER_NUMBER
	{
OVERLAY("通常"),JOUZAN("乗算"),SCREEN("スクリーン");
private static FILTER_NUMBER ar[] = FILTER_NUMBER.values();
private String viewString=null;
private FILTER_NUMBER(String str)
		{
viewString = str;
		}
@Override
public String toString()
		{
return viewString;
		}
public static FILTER_NUMBER getFilterNumberFromViewString(String compareString)
		{
FILTER_NUMBER ret=null;
for(FILTER_NUMBER i:ar)
			{
if(i.toString().equals(compareString))
				{
ret=i;
				}
			}
return ret;
		}
	}
private ArrayList<ImageCanvas> iclist = new ArrayList<ImageCanvas>();

private ScrollBar scrollX=null;
private ScrollBar scrollY=null;


private int realTimeDraggedX=0;
private int realTimeDraggedY=0;
private int draggedX=0;
private int draggedY=0;

private int beforeDraggingX=0;
private int beforeDraggingY=0;

private int pressedX=0;
private int pressedY=0;

private int realTimeReleasedX=0;
private int realTimeReleasedY=0;
private int releasedX=0;
private int releasedY=0;

private int locateX;
private int locateY;

private double pressure=1.0;
private double coefficient;
private double speedperframe;
private double acceralationperframe;
private double beforeacceralation;

private BufferedImage PileImage;
private BufferedImage drawingImage=null;
private DrawLineInfo dlinfo=new DrawLineInfo(0,1);
private int basethick=0;

private FILTER_NUMBER nowfilter=FILTER_NUMBER.OVERLAY;

private int nowlayer=0;

private boolean beforeMouseDragging=false;
private boolean mouseDraggingFlg;

private static int DEFAULT_WIDTH=400;
private static int DEFAULT_HEIGHT=400;

private CanvasPanel(String s,Component c,int... st)
	{
super(s,c,st);
	}
public CanvasPanel(Component c,ScrollBar scX,ScrollBar scY,int... st)
	{
super(c,st);
AwtPenToolkit.addPenListener(c,this);
scrollX=scX;
scrollY=scY;
try
		{
DEFAULT_WIDTH=Integer.parseInt(ConfigFileRW.ConfigList.LAYER_WIDTH.toString());
		}
catch(Exception e){;}
try
		{
DEFAULT_HEIGHT=Integer.parseInt(ConfigFileRW.ConfigList.LAYER_HEIGHT.toString());
		}
catch(Exception e){;}
setFrameFlg(false);
setTitleFlg(false);
setTabUseFlg(false);
addLayer();
	}
public void addLayer()
	{
addLayer(DEFAULT_WIDTH,DEFAULT_HEIGHT);
	}
public void addLayer(int width,int height)
	{
addLayer(makeBI(width,height));
	}
public void addLayer(BufferedImage bi)
	{
iclist.add(new ImageCanvas(bi));
refleshPileImage();
nowlayer=getLayerSize()-1;
	}
public void tradeLayer(int firstLayerNumber,int secondLayerNumber)
	{
notRefleshTradeLayer(firstLayerNumber,secondLayerNumber);
refleshPileImage();
	}
private void notRefleshTradeLayer(int firstLayerNumber,int secondLayerNumber)
	{
if(firstLayerNumber!=secondLayerNumber)
		{
ImageCanvas bf =iclist.set(firstLayerNumber,iclist.get(secondLayerNumber));
iclist.set(secondLayerNumber,bf);
		}
	}
/**
locateLayerとして指定された番号のレイヤとその一つ前の番号のレイヤとの間に新しいレイヤを幅width,高さheightとして作成します。
その際、locateLayerとして指定されたレイヤの番号は一つ上がり、新しく挿入されるレイヤの番号はlocateLayerと同じになります
*/
public void insertNewLayer(int width,int height,int locateLayer)
	{
insertNewLayer(makeBI(width,height),locateLayer);
	}
public void insertNewLayer(BufferedImage setLayerImage,int locateLayer)
	{
addLayer(setLayerImage);
insertLayer(getLayerSize()-1,locateLayer);
	}
/**
setLayerとして指定された番号のレイヤを、locateLayerとして指定された番号のレイヤとその一つ前のレイヤとの間に挿入します。
これを使用した後、レイヤが入れ替わるためレイヤ番号は変動します。
なお、0を指定した場合は最低位置のレイヤ（0番レイヤ、一番下のレイヤ）に、現在の最大レイヤ番号以上の数値を入れた場合最高位置のレイヤ((getNowLayerSize()-1)番レイヤ)となります。
*/
public void insertLayer(int setLayer,int locateLayer)
	{
int endLocate = locateLayer;
if(endLocate>getLayerSize())
		{
endLocate=getLayerSize()-1;
		}

if((0<=setLayer) && (setLayer<getLayerSize()) && (0<endLocate))
		{
if(setLayer>endLocate)
			{
for(int i=setLayer;i>endLocate;i--)
				{
notRefleshTradeLayer(i,i-1);
				}			
			}
else
			{
endLocate=endLocate-1;
for(int i=setLayer;i<endLocate;i++)
				{
notRefleshTradeLayer(i,i+1);
				}
			}
		}
refleshPileImage();
	}
public void removeLayer(int layernum)
	{
if(1<getLayerSize() && 0<=layernum && layernum<getLayerSize())
		{
iclist.remove(layernum);
setNowLayer(getNowLayerNumber());
refleshPileImage();
		}
	}
public void setnowfilter(String filternum)
	{
try
		{
nowfilter=FILTER_NUMBER.getFilterNumberFromViewString(filternum);
		}
catch(NullPointerException e)
		{
System.out.println("set failed filter:"+filternum);
		}
	}
public void refleshPileImage()
	{
PileImage=iclist.get(0).getNowImage();
for(int c=1;c<getLayerSize();c++)
		{
switch(nowfilter)
			{
case JOUZAN:
PileImage = ImageChenge.jouzan(PileImage,getImage(c));
break;
case SCREEN:
PileImage = ImageChenge.scr(PileImage,getImage(c));
break;
default://FILTER_OVERLAY
PileImage = ImageChenge.overlay(PileImage,getImage(c));
break;
			}
		}
scrollX.reset(PileImage.getWidth());
scrollY.reset(PileImage.getHeight());
	}
public void setNewImage(BufferedImage bi,int layernum)
	{
iclist.get(layernum).setNewImage(bi);
refleshPileImage();
	}
public void setNewImage(BufferedImage bi)
	{
setNewImage(bi,nowlayer);
	}
public void setImage(BufferedImage bi,int layernum)
	{
iclist.get(layernum).setImage(bi);
refleshPileImage();
	}
public void setImage(BufferedImage bi)
	{
setImage(bi,nowlayer);
	}
public BufferedImage getNowImage()
	{
return getImage(nowlayer);
	}
public BufferedImage getImage(int layernum)
	{
return iclist.get(layernum).getNowImage();
	}
public BufferedImage getPileImage()
	{
return PileImage;
	}
public ImageCanvas getLayer(int layernum)
	{
return iclist.get(layernum);	
	}
public ImageCanvas getNowLayer()
	{
return iclist.get(getNowLayerNumber());
	}
public BufferedImage getDrawingImage()
	{
return drawingImage;
	}
public void undo()
	{
iclist.get(getNowLayerNumber()).undo();
refleshPileImage();
	}
public void redo()
	{
iclist.get(getNowLayerNumber()).redo();
refleshPileImage();
	}
/**
 * 直接描画するためのBufferedImageインスタンスを作成し、それを返すためのメソッドです。
 * undo、redoに必要なので直接描画を行う場合はgetNowImage()に描画するのではなくこのメソッドから返されるインスタンスに描画してください。
 * undo、redoの区切りになる描画の度にこのメソッドを呼び出し、描画してください。
 * なお、変更が行われても表示される画像に影響はないので、描画が完了したら表示を更新する為にrefleshPileImage()の呼び出しを行ってください。
 * */
public BufferedImage preparationEditImage(int layernum)
	{
setNewImage(getImage(layernum));
return getImage(layernum);
	}
public BufferedImage preparationEditNowImage()
	{
return preparationEditImage(getNowLayerNumber());
	}
public int getNowLayerNumber()
	{
return nowlayer;
	}
public int getLayerSize()
	{
return iclist.size();
	}
public void setNowLayer(int n)
	{
int ret=n;
if(n<0)
		{
ret=0;
		}
if(getLayerSize()<=n)
		{
ret=getLayerSize()-1;
		}
nowlayer=ret;
	}
public static BufferedImage makeBI()
	{
return makeBI(DEFAULT_WIDTH,DEFAULT_HEIGHT);
	}
public static BufferedImage makeBI(int width,int height)
	{
BufferedImage ret = new BufferedImage(width,height,BufferedImage.TYPE_INT_ARGB);
for(int x=0;x<ret.getWidth();x++)
		{
for(int y=0;y<ret.getHeight();y++)
			{
ret.setRGB(x,y,0x00000000);
			}
		}
return ret;
	}
private void drawPileImage(Graphics g)
	{
if(getPileImage()!=null)
		{
int lx=locateX;
if(lx<0)
			{
lx=0;
			}
int ly=locateY;
if(ly<0)
			{
ly=0;
			}
int iw=getImageWidth();
if(iw>(getPileImage().getWidth()-lx))
			{
iw=getPileImage().getWidth()-lx;
			}
int ih=getImageHeight();
if(ih>(getPileImage().getHeight()-ly))
			{
ih=getPileImage().getHeight()-ly;
			}
if((iw>0) && (ih>0))
			{
g.drawImage(getPileImage().getSubimage(lx,ly,iw,ih),getImageX(),getImageY(),null);
if(mouseDraggingFlg && (getDrawingImage()!=null) && (gazoushori.getNowShoriMode()==gazoushori.SHORI_MODE.線を引く) && gazoushori.getSubPanel().getFreeLine())
				{
g.drawImage(getDrawingImage().getSubimage(lx,ly,iw,ih),getImageX(),getImageY(),null);
				}
			}
g.setColor(Color.black);
g.drawRect(getImageX()-1,getImageY()-1,iw+1,ih+1);
		}
else
		{
g.drawString("読み込み中",getX(),getY()+getHeight()/2);
		}
	}
private void drawBackGroundLine(Graphics g)
	{
g.setColor(Color.white);
g.fillRect(getX(),getY(), getX()+getWidth(), getY()+getHeight());
int linenum=20;
int lh=getX()+(getWidth()/linenum);
g.setColor(Color.LIGHT_GRAY);
for(int c=0;c<linenum;c++)
		{
g.drawLine(lh*c,0,lh*c,getY()+getHeight());
		}
linenum=(getHeight()/lh)+1;
for(int c=0;c<linenum;c++)
		{
g.drawLine(0, lh*c, getX()+getWidth(), lh*c);
		}
	}
@Override
public void usualPerform()
	{
if(mouseDraggingFlg)
		{
setMouseLocate();
		}
switch(gazoushori.getNowShoriMode())
		{
case 線を引く:
try
			{
if(gazoushori.getSubPanel().getFreeLine())
				{
usualLine();
				}
			}
catch(NullPointerException e){;}
break;
default:
break;
		}
	}
private void usualLine()
	{
if(mouseDraggingFlg)
		{
if(beforeMouseDragging)
			{
if(gazoushori.getSubPanel().getMoveThick())
				{
double bc=acceralationperframe-beforeacceralation;
//coefficient=1.0-(Math.abs(bc)/(Math.abs(bc)+speedperframe));
//加速度が増えたり減ったりするときに太くなる
if(Math.abs(bc)<Math.abs(speedperframe))//加速度差が減ると太くなる
					{
if(basethick>dlinfo.getThick())//太くなる
						{
if(coefficient<=0.5)
							{
coefficient=coefficient*2.0;
if(coefficient>0.4)//本来0.5になる時、誤差があるといけないので修正
								{
coefficient=0.5;
								}
							}
else
							{
coefficient=0.5+(coefficient/2.0);//(1.0-((1.0-coefficient)/2.0))の変形
							}
						}
					}
else if((((double)basethick)*coefficient)>1.0)//細くなる
					{
if(coefficient<=0.5)
						{
coefficient=coefficient/2.0;
						}
else
						{
coefficient=(-1.0)+(coefficient*2.0);//1.0-((1.0-coefficient)*2.0)の変形
if(coefficient<0.6)//本来0.5になる時、誤差があるといけないので修正
							{
coefficient=0.5;
							}
						}
					}
dlinfo.setThick((int)(coefficient*((double)(basethick*2))));
				}
else if(gazoushori.getSubPanel().getUsePentab())
				{
dlinfo.setThick((int)(((double)(basethick*2))*pressure));
				}
else
				{
dlinfo.setThick(basethick);
				}
if(gazoushori.getSubPanel().getAntiAliasMode())
				{
ImageChenge.drawAntiAliasLine(drawingImage,getDraggedX(),getDraggedY(),beforeDraggingX,beforeDraggingY,dlinfo);
				}
else
				{
ImageChenge.drawThickLine(drawingImage,getDraggedX(),getDraggedY(),beforeDraggingX,beforeDraggingY,dlinfo);
				}
			}
else
			{
if(ImageChenge.canColorTrans(gazoushori.getSubPanel().getColor1()))
				{
dlinfo.setColor(ImageChenge.colorTrans(gazoushori.getSubPanel().getColor1()));
				}
else
				{
dlinfo.setColor(0xff000000);
				}
coefficient=0.5;
try
				{
basethick=Integer.parseInt(gazoushori.getSubPanel().getThick());
				}
catch(Exception e)
				{
basethick=1;
				}
drawingImage=makeBI(getNowImage().getWidth(),getNowImage().getHeight());
			}
		}
else if(beforeMouseDragging)
		{
gazoushori.shori();
		}
beforeMouseDragging=mouseDraggingFlg;
beforeacceralation=acceralationperframe;
beforeDraggingX=getDraggedX();
beforeDraggingY=getDraggedY();
	}
@Override
public void drawAny(Graphics g)
	{
drawBackGroundLine(g);
drawPileImage(g);
	}
public int getImageX()
	{
return getX()+getWidth()/18;
	}
public int getImageY()
	{
return getY()+getHeight()/18;
	}
public int getImageWidth()
	{
return getWidth()*8/9;
	}
public int getImageHeight()
	{
return getHeight()*8/9;
	}
public int getDraggedX()
	{
return draggedX;
	}
public int getDraggedY()
	{
return draggedY;
	}
public int getPressedX()
	{
return pressedX;
	}
public int getPressedY()
	{
return pressedY;
	}
public int getReleasedX()
	{
return releasedX;
	}
public int getReleasedY()
	{
return releasedY;
	}
public boolean getMouseDraggingFlg()
	{
return mouseDraggingFlg;
	}
public void setMouseLocate()
	{
int dX=draggedX;
int dY=draggedY;
draggedX=locateX+realTimeDraggedX-getImageX();
draggedY=locateY+realTimeDraggedY-getImageY();
dX=dX-draggedX;
dY=dY-draggedY;
acceralationperframe=Math.sqrt((dX*dX)+(dY*dY))-speedperframe;
speedperframe=acceralationperframe+speedperframe;
	}
public void setImageLocation(int x,int y)
	{
locateX=x;
locateY=y;
	}
@Override
public void mouseDraggedPerform(MouseEvent e)
	{
realTimeDraggedX=e.getX();
realTimeDraggedY=e.getY();
	}
@Override
public void mousePressedPerform(MouseEvent e)
	{
if(containCheck(e.getX(),e.getY()))
		{
pressedX=locateX+e.getX()-getImageX();
pressedY=locateY+e.getY()-getImageY();
mouseDraggedPerform(e);
mouseDraggingFlg=true;
		}
	}
@Override
public void mouseReleasedPerform(MouseEvent e)
	{
releasedX=locateX+e.getX()-getImageX();
releasedY=locateY+e.getY()-getImageY();
mouseDraggingFlg=false;
	}
@Override
public void penKindEvent(PKindEvent e)
	{;}
@Override
public void penLevelEvent(PLevelEvent e)
	{
pressure=1.0;
for(PLevel level:e.levels)
		{
if(level.getType()==PLevel.Type.PRESSURE)
			{
pressure=level.value;
			}
		}
	}
@Override
public void penButtonEvent(PButtonEvent e)
	{;}
@Override
public void penScrollEvent(PScrollEvent e)
	{
	}
@Override
public void penTock(long avM)
	{;}
public void perform(){;}
}
