/*
Copyright (C) 2017 Hayakuchi

This file is part of HinesmPaint.

HinesmPaint is free software.
You can redistribute HinesmPaint and/or modify it under the GPL Licence version3 and LGPL Licence version 3.
(GPL text and LGPL text in http://www.gnu.org/licenses/)

このプログラムはフリーソフトです。
あなたはGPLライセンスversion3及びLGPLライセンスversion3の元でこのプログラムを改変、再配布することが可能です。
(GPLとLGPLの内容は http://www.gnu.org/licenses/)にあります)
*/
package hinesm.util.gazobutton;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import hinesm.util.UtilPerform;

/**
 * このクラスは、サブクラス毎に定義した処理をボタン操作によって実行するためのクラスです。
 * 抽象クラスになっていることからもわかりますが、基本的に拡張して使用します。
 * ボタンが操作されたかどうかは、使用先のクラスでMouseListener等を宣言する等して実行してください
 **/
public abstract class GazoButton {

// フィールド

// GazoButtonのリスト
private static ArrayList<GazoButton> btlist = new ArrayList<GazoButton>();
/*
 * このGazoButton上にさらに配置するGazoButtonは、
 * このCHILED_GAZOBOTTUN_LISTに追加すればこのGazoButtonをput(Graphics g)する際に自動的にこのGazoButtonの上にput(Graphics gされます)。
*/
private ArrayList<GazoButton> childgazobutton = new ArrayList<GazoButton>();
private GazoButton parentgazobutton = null;

// このボタンが設置されるコンポーネント
private Component dependcomponent = null;
/** デフォルトの幅(100) */
public static final int DEFAULT_WIDTH = 100;
/** デフォルトの高さ(50) */
public static final int DEFAULT_HEIGHT = 50;

private int x;
private int y;
private int width;
private int height;
private int newfontdotsize;
protected String[] titlestring;
/**
 * ボタンの色のデフォルトの値 インスタンス作成時にはデフォルトの値が設定されます
 */
public final Color DEFAULT_BOTTUN_COLOR = new Color(0xffffffff, true);
private Color buttoncolor = new Color(DEFAULT_BOTTUN_COLOR.getRGB(), true);
/**
 * フレームの色のデフォルトの値 インスタンス作成時にはデフォルトの値が設定されます
 */
public final Color DEFAULT_FRAME_COLOR = new Color(0xff000000, true);
private Color framecolor = new Color(DEFAULT_FRAME_COLOR.getRGB(), true);
/**
 * 文字の色のデフォルトの値 インスタンス作成時にはデフォルトの値が設定されます
 */
public final Color DEFAULT_TITLE_COLOR = new Color(0xff000000, true);
private Color titlecolor = new Color(DEFAULT_FRAME_COLOR.getRGB(), true);

/**
 * 表示文字のフォントです 取得、設定にはgetTitleFont及びsetTitleFontを使用します。
 */
protected Font titlefont;
/**
 * 表示文字のフォントのデフォルトのフォント名です フォント名は論理フォントMONOSPACEDです。しかし使用言語によっては文字コードが対応していない場合があります。実際に使用されるデフォルトのフォント名は、getDefaultFontString()で取得してください。
 */
private static final String DEFAULT_FONT_STRING = Font.MONOSPACED;
private static String TRUTH_DEFAULT_FONT_STRING=DEFAULT_FONT_STRING;//実際に使用するデフォルトのフォントです。表示予定の文字に言語が対応してない場合にSetUseLanguage(int langNumber)で設定します。
public static enum UseLanguage
	{
ENGLISH,JAPANESE
	}

/**
 * 表示文字のフォントのデフォルトのフォントスタイルです フォントスタイルはプレーンスタイルです
 */
public static final int DEFAULT_FONT_STYLE = Font.PLAIN;
/**
 * 表示文字のフォントのデフォルトのサイズです フォントサイズは10ですが、コンストラクタでボタンのサイズに合わせた値を定義します
 */
public static final int DEFAULT_FONT_SIZE = 10;
/*
 * Tabキーによってボタンを選んで押すことができる機能の実装を検討中で、その場合そのために必要だと思われる変数です 現時点においては無意味な変数です
 */
public static final boolean TAB_MOVE_POJI = true;
public static final boolean TAB_USE_NEGA = false;
private boolean tabuseflg;
private static int tabnow;
private int tabnum;
private static boolean shiftflg;

/**
 * ボタンに表示される文字です。 インスタンス作成時に指定しない場合デフォルトの文字が設定されます
 */
protected StringBuilder title;
/**
 * 依存するコンポーネントのgetColor()で得られる色。put時に色を元に戻す際に使います
 */
public Color gc;
/**
 * 依存するコンポーネントのgetFont()で得られるフォント。put時に元に戻す際に使います
 */
public Font gf;

private boolean frameflg;
private boolean titleflg;
private boolean activeflg;

// コンストラクタ

/**
 * 引数なしのコンストラクタを作成することはできません。 必ず位置とこのボタンが設置されるコンポーネントの設定をしてください。
 * 幅と高さとボタンの文字の内容は任意で設定することができます。
 */
private GazoButton()
	{
	}
/**
 * このコンストラクタによって、作成されたインスタンスの位置が(x,y)、幅がwidth、高さがheightに設定されます。
 * また、ボタンの内容がsに設定されます。
 */
public GazoButton(Component c, int... stetus)
	{
this("title", c, stetus);
	}

public GazoButton(String s, Component c, int... stetus)
	{
this();
if ((stetus.length == 2) || (stetus.length == 3))
		{
constracter(s, c, stetus[0], stetus[1]);
		}
else if (stetus.length >= 4)
		{
constracter(s, c, stetus[0], stetus[1], stetus[2], stetus[3]);
		}
	}

public void constracter(String s, Component c, int x, int y)
	{
constracter(s, c, x, y, DEFAULT_WIDTH, DEFAULT_HEIGHT);
	}

public void constracter(String s, Component c, int x, int y, int width, int height)
	{
setDependComponent(c);
setStetus(x, y, width, height);
setTitle(s);
setTitleFont(new Font(TRUTH_DEFAULT_FONT_STRING, DEFAULT_FONT_STYLE,UtilPerform.getFontSize(difineFontDot(s, getWidth(), getHeight()))));
setFrameFlg(true);
setTitleFlg(true);
setActiveFlg(true);
setTabUseFlg(true);
tabnum = btlist.size();
btlist.add(this);
	}


// difine fontdotsize
public int difineFontDot(String s, int gW, int gH)
	{
newfontdotsize = 1;
if (s.length() > 0)
		{
int Wpl=(gW/(s.length()));
int Hl = (gH*(s.length()));
if (Wpl <= gH)
			{
if (Hl <= gW)
				{
newfontdotsize = gH;
				}
else
				{
newfontdotsize = Wpl;
				}
			}
else
			{
newfontdotsize = gH;
			}
		}
else
		{
newfontdotsize = gH;
		}
return newfontdotsize;
	}

public int getFontDot()
	{
return newfontdotsize;
	}
public static String getDefaultFontString()
	{
return TRUTH_DEFAULT_FONT_STRING;
	}
// フィールドの設定

/**
 * このボタンの設置されるコンポーネントを設定します
 **/
private void setDependComponent(Component c)
	{
dependcomponent = c;
	}

/**
 * このボタンに表示される名前を設定します。
 **/

public void setTitle(String s)
	{
if (title != null)
		{
if (!(s.equals(title)))
			{
title.replace(0, title.length(), s);
makeString();
			}
		}
else
		{
title = new StringBuilder(s);
makeString();
		}
	}

private void makeString()
	{
titlestring = new String[title.length()];
int cou = 0;
while (titlestring.length > cou)
		{
titlestring[cou] = title.substring(cou, cou + 1);
cou = cou + 1;
		}
	}

/**
 * このボタンの位置を設定します。
 */

public void setLocation(int x, int y)
	{
this.x = x;
this.y = y;
	}

/**
 * このボタンの幅と高さを設定します。
 */

public void setSize(int width, int height)
	{
this.width = width;
this.height = height;
	}

/**
 * このボタンの位置と幅と高さを設定します。
 */
public void setStetus(int x, int y, int width, int height)
	{
setLocation(x, y);
setSize(width, height);
	}

/**
 * 引数のaとcの色が一致していた場合、aを返し、 一致していなかった場合cの色をコピーして返します。
 */
public static final Color translateColor(Color c, Color a)
	{
Color ret;
if (c.getRGB() != a.getRGB())
		{
ret = new Color(c.getRGB(), true);
		}
else
		{
ret = a;
		}
return ret;
	}

/**
 * このボタンの色をARGB値で設定します。 引数のaflgがtrueの場合アルファ値をそのままに、
 * falseの場合アルファ値をffとして設定します（つまり内部的には透明度が設定された色ですが、数値的には透明度0なので非透明と同じ状態にします）。
 * 新しいColorインスタンスを作成するので高速処理には不向きです。
 * より詳細な内容が知りたい場合は、java.awt.ColorクラスのColor(int rgba,boolean hasalpha)コンストラクタを確認してください（このメソッドのソースは、ボタンの色に、java.awt.Colorクラスの新しいインスタンスを上記の引数で代入しているのみです）。
 */

public void setColor(int n, boolean aflg)
	{
if(differentColor(buttoncolor,n,aflg))
		{
buttoncolor = new Color(n, aflg);
		}
	}

/**
 * このボタンの色をColorインスタンスで設定します。
 * こちらは代入された色のsRGB値と元々のこのボタンの色のsRGB値が一致していない限り新しくインスタンスを作成することがないので比較的高速処理に向いています。
 */
public void setColor(Color c)
	{
buttoncolor = translateColor(c, buttoncolor);
	}
/**
 * このボタンの文字色をARGB値で設定します。 対象とするColor型フィールド（文字の色を示すフィールドです）以外はsetColor(int
 * n,boolean aflg)と同じなので詳細はそちらを確認してください。
 */

public void setTitleColor(int n, boolean aflg)
	{
if(differentColor(titlecolor,n,aflg))
		{
titlecolor = new Color(n, aflg);
		}
	}

/**
 * このボタンの文字色をARGB値で設定します。 対象とするColor型フィールド（文字の色を示すフィールドです）以外はsetColor(Color
 * c)と同じなので詳細はそちらを確認してください。
 */
public void setTitleColor(Color c)
	{
titlecolor = translateColor(c, titlecolor);
	}

/**
 * このボタンの枠のをARGB値で設定します。 対象とするColor型フィールド（枠の色を示すフィールドです）以外はsetColor(int n,boolean aflg)と同じなので詳細はそちらを確認してください。
*/

public void setFrameColor(int n, boolean aflg)
	{
if(differentColor(framecolor,n,aflg))
		{
framecolor = new Color(n, aflg);
		}
	}

/**
 * このボタンの文字色をARGB値で設定します。 対象とするColor型フィールド（文字の色を示すフィールドです）以外はsetColor(COlor
 * c)と同じなので詳細はそちらを確認してください。
 */
public void setFrameColor(Color c)
	{
framecolor = translateColor(c, framecolor);
	}
private boolean differentColor(Color c,int rgb,boolean hasalpha)
	{
return c.getRGB()!=rgb;
	}

/**
 * このボタンに表示される文字のフォントを設定します
 */
public void setTitleFont(Font f)
	{
Font ff;
if (f.getSize() > UtilPerform.getFontSize(difineFontDot(getTitle(), getWidth(), getHeight())))
		{
ff = new Font(f.getName(), f.getStyle(),UtilPerform.getFontSize(difineFontDot(getTitle(), getWidth(), getHeight())));
		}
		else
		{
ff = f;
		}
titlefont = ff;
	}

/**
 * このボタンの背景色、枠が表示されるかを設定します。
 */
public void setFrameFlg(boolean b)
	{
frameflg = b;
	}

/**
 * このボタンの文字が表示されるかを設定します。
 */
public void setTitleFlg(boolean b)
	{
titleflg = b;
	}

/**
 * このボタンの表示、マウスやキー、その他処理等が有効になるかどうかを設定します。 bがtrueならば有効、falseならば無効です。
 * 有効な場合を「アクティブである」と表記します(無効な場合は「非アクティブ」と表記します)。
 */
private void setActiveFlgChild(boolean b)
	{
activeflg=b;
if(childgazobutton.size()>0)
		{
int c=0;
while(c<childgazobutton.size())
			{
childgazobutton.get(c).setActiveFlgChild(b);
c=c+1;
			}
		}
	}
public void setActiveFlg(boolean b)
	{
if(childgazobutton.size()<=0)
		{
activeflg=b;
		}
else
		{
setActiveFlgChild(b);
		}
	}
/**
 * 引数となったコンポーネントインスタンスに依存しているインスタンスの中で、
 * かつ引数となったGazoButtonを全てアクティブにしGazoButtonは全て非アクティブにします。
 */
public static final void setActiveAll(Component dcomp, GazoButton... gb) 
	{
setActiveFlgAll(dcomp,false,btlist);
setActiveFlgAll(dcomp,true,gb);
	}
private static final void setActiveFlgAll(Component dcomp,boolean b,ArrayList<GazoButton> gb)
	{
int c=0;
while(c<gb.size())
		{
if(gb.get(c)!=null && gb.get(c).getDependComponent()==dcomp)
			{
gb.get(c).setActiveFlg(b);
			}
c=c+1;
		}
	}
private static final void setActiveFlgAll(Component dcomp,boolean b,GazoButton... gb)
	{
int c=0;
while(c<gb.length)
		{
if(gb[c]!=null && gb[c].getDependComponent()==dcomp)
			{
gb[c].setActiveFlg(b);
			}
c=c+1;
		}
	}
/**
 * dcompに依存しているGazoButtonを全てアクティブにします。
 */
public final void setActiveAll(Component dcomp)
	{
int c = 0;
while (c < btlist.size())
		{
if (btlist.get(c) != null && btlist.get(c).getDependComponent() == dcomp)
			{
btlist.get(c).setActiveFlg(true);
			}
c = c + 1;
		}
	}

public void setTabUseFlg(boolean b)
	{
tabuseflg = b;
	}

public static final boolean setTabNow(int n)
	{
if (0 <= n && n < btlist.size() && btlist.get(n) != null && btlist.get(n).getActiveFlg() && btlist.get(n).getTabUseFlg())
		{
tabnow = n;
return true;
		}
else
		{
return false;
		}
	}

public static final void moveTabNow(boolean b)
	{
int c = 0;
boolean tabflg = false;
while (c < btlist.size())
		{
if (btlist.get(c) != null && btlist.get(c).getActiveFlg() && btlist.get(c).getTabUseFlg())
			{
tabflg = true;
c = btlist.size();
			}
c = c + 1;
		}
c = getTabNow();
int inc = 1;
if (!(b))
		{
inc = -1;
		}
while (tabflg)
		{
c = ((c + inc) % (btlist.size()));
if (c < 0)
			{
c = (btlist.size() - 1) + c;
			}
tabflg = !(btlist.get(c) != null && setTabNow(c));
		}
	}
/**
使用言語を設定します。
ボタンの表示は使用言語に合わせたフォントとなります。
@return 対応するフォントが見つかった場合trueを返します。
*/
public static boolean setUseLanguage(UseLanguage u)
	{
char searchLangChar='A';
switch(u)
		{
case JAPANESE:
searchLangChar='墨';
break;
default:
break;
		}
Font[] f=java.awt.GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts();
for(int i=f.length-1;i>=0;i--)
		{
if(f[i].canDisplay(searchLangChar))
			{
TRUTH_DEFAULT_FONT_STRING=f[i].getName();
return true;
			}
		}
return false;
	}
/**
 * このボタン上に設置するGazoButtonを登録します。 登録を行うことでsetActiveFlgAll(Component
 * c,Gazobutton[] gb)を行うときに子GazoButtonを追加する必要がなくなります。
 * 逆に言えばGazoButton上に別のGazoButtonを設置するという構造のスーパークラスを定義する場合、
 * 隠蔽を行っていたならばsetActiveFlgAllを使用した場合、設置されたGazoButtonも非アクティブになってしまうため、
 * その後外部からsetActiveFlg(boolean b)を呼び出すことができないので表示が不可能になります。
 */
public void addChildGazoButton(GazoButton childgb)
	{
childgazobutton.add(childgb);
childgb.setParentGazoButton(this);
	}

private void setParentGazoButton(GazoButton parentgb)
	{
parentgazobutton = parentgb;
	}
// フィールドの取得

/**
 * このボタンのx座標を返します
 */

public final int getX()
	{
return x;
	}

/**
 * このボタンのy座標を返します
 */

public final int getY()
	{
return y;
	}

/**
 * このボタンの幅を返します
 */

public final int getWidth()
	{
return width;
	}

/**
 * このボタンの高さを返します
 */

public final int getHeight()
	{
return height;
	}

/**
 * このボタンに表示される文字を返します
 */

public final String getTitle()
	{
return title.toString();
	}

/**
 * このボタンの設置されているコンポーネントを返します
 */

public final Component getDependComponent()
	{
return dependcomponent;
	}

/**
 * このボタンの枠内の背景色を返します
 */

public final Color getColor()
	{
return buttoncolor;
	}

/**
 * このボタンに表示される文字の色を返します
 */

public final Color getTitleColor()
	{
return titlecolor;
	}

/**
 * このボタンの枠の色を返します
 */

public final Color getFrameColor()
	{
return framecolor;
	}

/**
 * このボタンのタイトルのフォントを返します
 */
public final Font getTitleFont()
	{
return titlefont;
	}

/**
 * このボタンの枠、背景色が表示されるかを返します。
 */
public final boolean getFrameFlg()
	{
return frameflg;
	}

/**
 * このボタンの文字が表示されるかを返します。
 */
public final boolean getTitleFlg()
	{
return titleflg;
	}

/**
 * このボタンの表示、マウスやキー入力、その他処理等が有効になっているかどうかを返します。
 */
public final boolean getActiveFlg()
	{
return activeflg;
	}

public final boolean getTabUseFlg()
	{
return tabuseflg;
	}

public int getTabNum()
	{
return tabnum;
	}

public static final int getTabNow()
	{
return tabnow;
	}

public final boolean thisTabNow()
	{
return GazoButton.getTabNow() == this.getTabNum();
	}

private GazoButton getParentGazoButton()
	{
return parentgazobutton;
	}







	// ここから先は描画関連






/**
 * 引数に指定したGraphicsインスタンスで現在使用しているgetFont()及びgetColor()のコピーを、
 * このクラスのインスタンスのフィールドに新しく作成、代入します。
 * これを用いることでputの時にGraphicsのsetFont()及びsetColor()で元に戻すことが可能となります。
 * (ただし、put()またはput(Graphics g)の実行時にはこのgetInfo(Graphics
 * g)メソッドを呼び出したされた時点のFont及びColorを代入するので、put()を呼び出す前にsetColor()などで色を変更していた場合
、期待していた処理は行えない（つまり元に戻すのではなく別の情報を代入してしまう）可能性があります)
	 */
public void getGInfo(Graphics g)
	{
if (gf == null)
		{
gf = new Font(new String(g.getFont().getName()), g.getFont().getStyle(), g.getFont().getSize());
		}
if ((!(g.getFont().getName().equals(gf.getName()))) || g.getFont().getStyle() != gf.getStyle() || g.getFont().getSize() != gf.getSize())
		{
gf = new Font(new String(g.getFont().getName()), g.getFont().getStyle(), g.getFont().getSize());
		}

if (gc == null)
		{
gc = new Color(g.getColor().getRGB(), true);
		}
if (g.getColor().getRGB() != gc.getRGB())
		{
gc = new Color(g.getColor().getRGB(), true);
		}

	}

/**
 * 引数となったコンポーネントに依存しているGazoButtonを全て引数gに描画します。
 */
public static final void putAll(Graphics g, Component putplace)
	{
usualPerformAll(putplace);
for(int c=0;c<btlist.size();c++)
		{
if (btlist.get(c) != null && btlist.get(c).getDependComponent() == putplace)
			{
btlist.get(c).put(g);
			}
		}
	}

/**
 * このメソッドは、引数となったGraphicsインスタンスにこのボタンを描画するためのクラスです。
 * 描画するのみなので押してもマウスを移動させても何の反応もありません。
 * マウスに関する処理を行う場合、mouse*PerformAll(MouseEvent
 * e)をMouseListenerインターフェースを実装したコンポーネントで実行してください
 **/
public final void put(Graphics g)
	{
if (activeflg)
		{
try
			{
getGInfo(g);
if (frameflg)
				{
drawFrame(g);
				}
if (titleflg)
				{
drawTitle(g);
				}
if (thisTabNow() && getTabUseFlg())
				{
drawTabFrame(g);
				}
drawAny(g);
// 元の色に戻します
g.setColor(gc);
			}
catch (Exception e)
			{
e.printStackTrace();
			}
		}
	}

/**
 * putメソッド内で描画処理を行う際にオーバーライドするメソッドです。
 * putメソッドはfinalメソッドなので、オーバーライドすることができません。
 * なので、putメソッド内で行いたい特殊な描画処理がある場合、このメソッドをオーバーライドし、使用してください。
 */
public void drawAny(Graphics g)
	{
;
	}

/**
 * GazoButtonにおけるTabが自身のインスタンスを指している場合、その時に描画される枠です。
 */
public void drawTabFrame(Graphics g)
	{
g.setColor(getFrameColor());
g.drawRect(x + 1, y + 1, getWidth() - 2, getHeight() - 2);
	}

/**
 * ボタンの枠を描画するためのメソッドです。
 * getFrameFlg()で得られる戻り値が、falseの場合この部分の処理は行われずtrueの場合行われます。
 * getFrameFlg()で得られる戻り値を変更したい場合、setFrameFlg(boolean b)を使用してください
 */
public void drawFrame(Graphics g)
	{
// 枠の内部を描きます
g.setColor(this.getColor());
g.fillRect(x+ 1, y + 1, width - 1, height - 1);
// 枠を描きます
g.setColor(this.getFrameColor());
g.drawLine(x,y+1,x,y+height-1);// 左辺
g.drawLine(x+1,y,x+width-1,y);// 上辺
g.drawLine(x+width,y+1,x+width,y+height-1);// 右辺
g.drawLine(x+1,y+height,x+width-1,y+height);// 底辺
	}

/**
 * ボタン内に文字を描画するためのメソッドです。
 * getTitle()メソッドで得られる文字列を、枠内に収まるようなサイズに変換されたサイズで描画を行います。
 * getTitleFlg()で得られる戻り値が、falseの場合この部分の処理は行われずtrueの場合行われます。
 * getTitleFlg()で得られる戻り値を変更したい場合、setTitleFlg(boolean b)を使用してください
 */
public void drawTitle(Graphics g)
	{
// 文字を描きます
g.setFont(this.getTitleFont());
g.setColor(this.getTitleColor());
if (titlestring != null)
		{
for(int cou=0;titlestring.length > cou;cou++)
			{
try
				{
g.drawString(titlestring[cou],x+getFontDot()*cou,y+(getHeight()*3/4));
				}
catch(Exception e)
				{
System.out.println(cou+":"+titlestring[cou]);
e.printStackTrace();
				}
			}
		}
g.setFont(gf);
	}
// ボタンの内容の実行関連
private static void usualPerformAll(Component com)
	{
int c = 0;
while (c < btlist.size())
		{
			if ((btlist.get(c) != null) && btlist.get(c).getActiveFlg() && (btlist.get(c).getDependComponent() == com))
			{
btlist.get(c).usualPerform();
			}
c = c + 1;
		}
	}

public void usualPerform()
	{
;
	}

private static void mouseEventPerformAll(MouseEvent e, int t)
	{
int c = 0;
while (c < btlist.size())
		{
if (btlist.get(c) != null && btlist.get(c).getActiveFlg() && e.getComponent() == btlist.get(c).getDependComponent())
			{
switch (t)
				{
case 1:
if (btlist.get(c).containCheck(e.getX(), e.getY()) && btlist.get(c).getTabUseFlg())
					{
setTabNow(btlist.get(c).getTabNum());
					}
btlist.get(c).mousePressedPerform(e);
break;
case 2:
btlist.get(c).mouseEnteredPerform(e);
break;
case 3:
btlist.get(c).mouseExitedPerform(e);
break;
case 4:
btlist.get(c).mouseClickedPerform(e);
break;
case 5:
btlist.get(c).mouseMovedPerform(e);
break;
case 6:
btlist.get(c).mouseDraggedPerform(e);
break;
default:
btlist.get(c).mouseReleasedPerform(e);
break;
				}
			}
c = c + 1;
		}
	}

public static final void processMouseEventAll(MouseEvent e)
	{
int id = 0;
switch (e.getID())
		{
case MouseEvent.MOUSE_PRESSED:
id = 1;
break;
case MouseEvent.MOUSE_ENTERED:
id = 2;
break;
case MouseEvent.MOUSE_EXITED:
id = 3;
break;
case MouseEvent.MOUSE_CLICKED:
id = 4;
break;
case MouseEvent.MOUSE_MOVED:
id = 5;
break;
case MouseEvent.MOUSE_DRAGGED:
id = 6;
break;
default:
// id=MouseEvent.MOUSE_RELEASED//表記の必要無し
break;
		}
mouseEventPerformAll(e, id);
	}

public static final void mouseReleasedPerformAll(MouseEvent e)
	{
mouseEventPerformAll(e, 0);
	}

public void mouseReleasedPerform(MouseEvent e)
	{
;
	}

public static final void mousePressedPerformAll(MouseEvent e)
	{
mouseEventPerformAll(e, 1);
	}

	public void mousePressedPerform(MouseEvent e)
	{
;
	}

public static final void mouseEnteredPerformAll(MouseEvent e)
	{
		mouseEventPerformAll(e, 2);
	}

	public void mouseEnteredPerform(MouseEvent e)
	{
;
	}

public static final void mouseExitedPerformAll(MouseEvent e)
	{
mouseEventPerformAll(e, 3);
	}

public void mouseExitedPerform(MouseEvent e)
	{
;
	}

public static final void mouseClickedPerformAll(MouseEvent e)
	{
mouseEventPerformAll(e, 4);
	}

public void mouseClickedPerform(MouseEvent e)
	{
;
	}

public static final void mouseMovedPerformAll(MouseEvent e)
	{
mouseEventPerformAll(e, 5);
	}

public void mouseMovedPerform(MouseEvent e)
	{
;
	}

public static final void mouseDraggedPerformAll(MouseEvent e)
	{
mouseEventPerformAll(e, 6);
	}

public void mouseDraggedPerform(MouseEvent e)
	{
;
	}

private static void keyEventPerformAll(KeyEvent e, int t)
	{
int c = 0;
while (c < btlist.size())
		{
if ((btlist.get(c) != null) && btlist.get(c).getActiveFlg())
			{
switch (t)
				{
case 1:
btlist.get(c).keyReleasedPerform(e);
break;
case 2:
btlist.get(c).keyTypedPerform(e);
break;
default:
btlist.get(c).keyPressedPerform(e);
break;
				}
			}
c = c + 1;
		}
	}

public static final void keyPressedPerformAll(KeyEvent e)
	{
if (e.getKeyCode() == KeyEvent.VK_SHIFT)
		{
shiftflg = true;
		}
if (shiftflg)
		{
switch (e.getKeyCode())
			{
case KeyEvent.VK_RIGHT:
moveTabNow(true);
break;
case KeyEvent.VK_LEFT:
moveTabNow(false);
break;
default:
keyEventPerformAll(e, 0);
break;
			}
		}
else
		{
keyEventPerformAll(e, 0);
		}
	}

public void keyPressedPerform(KeyEvent e)
	{
;
	}

public static final void keyReleasedPerformAll(KeyEvent e)
	{
if (e.getKeyCode() == KeyEvent.VK_SHIFT)
		{
shiftflg = false;
		}
if (shiftflg)
		{
switch (e.getKeyCode())
			{
case KeyEvent.VK_RIGHT:
break;
case KeyEvent.VK_LEFT:
break;
default:
keyEventPerformAll(e, 1);
break;
			}
		}
		else
		{
keyEventPerformAll(e, 1);
		}
	}

public void keyReleasedPerform(KeyEvent e)
	{
;
	}

public static final void keyTypedPerformAll(KeyEvent e)
	{
keyEventPerformAll(e, 2);
	}

public void keyTypedPerform(KeyEvent e)
	{
;
	}

/**
 * このメソッドはマウスでの操作を想定して作成したものです。
 * このメソッドを実行したGazoButtonインスタンスの(x,y,width,height)領域の中に(mx,my)
 * で指定した位置が存在すればtrueを返します。
 **/

public boolean containCheck(int mx, int my)
	{
return (mx >= getX() && mx <= getX() + getWidth() && my >= getY() && my <= getY() + getHeight());
	}

/**
 * このメソッドはサブクラスでオーバーライドし、、 このクラスのサブクラスで定義された処理を実行します。
 **/
public abstract void perform();

/**
 * このメソッドは、引数がtrueの場合、上記のperform()で定義された処理を実行します。 このメソッドの引数にcontainCheck(int
 * mx,int my)を用いる場合は好みによってifContainPerformを使いわけるとよいでしょう。
 **/

public void perform(boolean b)
	{
if(b)
		{
perform();
		}
	}

/**
 * このメソッドは指定された位置（x,y）が、ボタンの領域に収まっている場合perform()を実行します。
 * すっきりしたコードにしたい場合にperform(containCheck(int mx,int my))と使い分けるとよいでしょう
 * (ソースを見ると分かりますが中のコードは上記を実行しているのみです。)
 **/

public void ifContainPerform(int mx, int my)
	{
perform(containCheck(mx, my));
	}
}
